From 6477ce2aa8540fedbec7ebba5656cd3968e78019 Mon Sep 17 00:00:00 2001
From: James Moger <james.moger@gitblit.com>
Date: Wed, 02 Nov 2011 19:25:49 -0400
Subject: [PATCH] Populate author selector based on repository selector
---
src/com/gitblit/utils/SyndicationUtils.java | 104 ++++++++++++++++++++++++++++++++++++++++++++-------
1 files changed, 89 insertions(+), 15 deletions(-)
diff --git a/src/com/gitblit/utils/SyndicationUtils.java b/src/com/gitblit/utils/SyndicationUtils.java
index fb40dd5..8987c26 100644
--- a/src/com/gitblit/utils/SyndicationUtils.java
+++ b/src/com/gitblit/utils/SyndicationUtils.java
@@ -16,15 +16,17 @@
package com.gitblit.utils;
import java.io.IOException;
+import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
+import java.net.URLConnection;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
-import org.eclipse.jgit.revwalk.RevCommit;
-
import com.gitblit.Constants;
+import com.gitblit.GitBlitException;
+import com.gitblit.models.SyndicatedEntryModel;
import com.sun.syndication.feed.synd.SyndContent;
import com.sun.syndication.feed.synd.SyndContentImpl;
import com.sun.syndication.feed.synd.SyndEntry;
@@ -33,7 +35,9 @@
import com.sun.syndication.feed.synd.SyndFeedImpl;
import com.sun.syndication.feed.synd.SyndImageImpl;
import com.sun.syndication.io.FeedException;
+import com.sun.syndication.io.SyndFeedInput;
import com.sun.syndication.io.SyndFeedOutput;
+import com.sun.syndication.io.XmlReader;
/**
* Utility class for RSS feeds.
@@ -44,22 +48,24 @@
public class SyndicationUtils {
/**
* Outputs an RSS feed of the list of commits to the outputstream.
*
* @param hostUrl
* @param title
* @param description
* @param repository
- * @param commits
+ * @param entryModels
* @param os
* @throws IOException
* @throws FeedException
*/
public static void toRSS(String hostUrl, String title, String description, String repository,
- List<RevCommit> commits, OutputStream os) throws IOException, FeedException {
+ List<SyndicatedEntryModel> entryModels, OutputStream os) throws IOException,
+ FeedException {
SyndFeed feed = new SyndFeedImpl();
feed.setFeedType("rss_2.0");
+ feed.setEncoding("UTF-8");
feed.setTitle(title);
feed.setLink(MessageFormat.format("{0}/summary/{1}", hostUrl,
StringUtils.encodeURL(repository)));
@@ -71,26 +77,94 @@
feed.setImage(image);
List<SyndEntry> entries = new ArrayList<SyndEntry>();
- for (RevCommit commit : commits) {
+ for (SyndicatedEntryModel entryModel : entryModels) {
SyndEntry entry = new SyndEntryImpl();
- entry.setTitle(commit.getShortMessage());
- entry.setAuthor(commit.getAuthorIdent().getName());
- entry.setLink(MessageFormat.format("{0}/commit/{1}/{2}", hostUrl,
- StringUtils.encodeURL(repository), commit.getName()));
- entry.setPublishedDate(commit.getCommitterIdent().getWhen());
+ entry.setTitle(entryModel.title);
+ entry.setAuthor(entryModel.author);
+ entry.setLink(entryModel.link);
+ entry.setPublishedDate(entryModel.published);
SyndContent content = new SyndContentImpl();
- content.setType("text/html");
- String html = StringUtils.escapeForHtml(commit.getFullMessage(), false);
- content.setValue(StringUtils.breakLinesForHtml(html));
+ content.setType(entryModel.contentType);
+ content.setValue(entryModel.content);
entry.setDescription(content);
entries.add(entry);
}
feed.setEntries(entries);
- OutputStreamWriter writer = new OutputStreamWriter(os);
+ OutputStreamWriter writer = new OutputStreamWriter(os, "UTF-8");
SyndFeedOutput output = new SyndFeedOutput();
output.output(feed, writer);
writer.close();
}
+
+ /**
+ * Reads a Gitblit RSS feed.
+ *
+ * @param url
+ * the url of the Gitblit server
+ * @param repository
+ * the repository name
+ * @param branch
+ * the branch name (optional)
+ * @param numberOfEntries
+ * the number of entries to retrieve. if <= 0 the server default
+ * is used.
+ * @param username
+ * @param password
+ * @return a list of SyndicationModel entries
+ * @throws {@link IOException}
+ */
+ public static List<SyndicatedEntryModel> readFeed(String url, String repository, String branch,
+ int numberOfEntries, String username, char[] password) throws IOException {
+ // build feed url
+ List<String> parameters = new ArrayList<String>();
+ if (numberOfEntries > 0) {
+ parameters.add("l=" + numberOfEntries);
+ }
+ if (!StringUtils.isEmpty(branch)) {
+ parameters.add("h=" + branch);
+ }
+ StringBuilder sb = new StringBuilder();
+ sb.append(MessageFormat.format("{0}" + Constants.SYNDICATION_PATH + "{1}", url, repository));
+ if (parameters.size() > 0) {
+ boolean first = true;
+ for (String parameter : parameters) {
+ if (first) {
+ sb.append('?');
+ first = false;
+ } else {
+ sb.append('&');
+ }
+ sb.append(parameter);
+ }
+ }
+ String feedUrl = sb.toString();
+
+ URLConnection conn = ConnectionUtils.openReadConnection(feedUrl, username, password);
+ InputStream is = conn.getInputStream();
+ SyndFeedInput input = new SyndFeedInput();
+ SyndFeed feed = null;
+ try {
+ feed = input.build(new XmlReader(is));
+ } catch (FeedException f) {
+ throw new GitBlitException(f);
+ }
+ is.close();
+ List<SyndicatedEntryModel> entries = new ArrayList<SyndicatedEntryModel>();
+ for (Object o : feed.getEntries()) {
+ SyndEntryImpl entry = (SyndEntryImpl) o;
+ SyndicatedEntryModel model = new SyndicatedEntryModel();
+ model.repository = repository;
+ model.branch = branch;
+ model.title = entry.getTitle();
+ model.author = entry.getAuthor();
+ model.published = entry.getPublishedDate();
+ model.link = entry.getLink();
+ model.content = entry.getDescription().getValue();
+ model.contentType = entry.getDescription().getType();
+ entries.add(model);
+ }
+ return entries;
+ }
}
@@ -44,22 +48,24 @@
public class SyndicationUtils {
/**
- * Outputs an RSS feed of the list of commits to the outputstream.
+ * Outputs an RSS feed of the list of entries to the outputstream.
*
* @param hostUrl
--
Gitblit v1.9.1